package adm

import (
	"context"
	"encoding/json"
	"fmt"
	"gd_vehicle/apis"
	"gd_vehicle/errors"
	"gd_vehicle/rpc_apis"
	dutils "gd_vehicle/utils"
	"strconv"
	"time"

	"gd_vehicle/common.in/jsonrpc2"
	"gd_vehicle/common.in/utils"
	"go.uber.org/zap"
)

func CheckProviderApiIsAvaliable(code string, providerMap map[string]apis.MerchantProviderLimitInfo, rangeNo int) error {
	v, ok := providerMap[code+strconv.Itoa(rangeNo)]
	if ok {
		ret, _ := dutils.CheckProviderAvailable(v)
		if ret == false {
			return errors.ProviderUnavailable
		}
	} else {
		return errors.ProviderApiNotFound

	}
	return nil
}

type Adm struct {
	Ctx      context.Context
	Params   map[string]string
	Code     string
	LReqs    *[]apis.ThirdpartLogWrite
	Provider apis.MerchantProviderLimitInfo
}

func (a *Adm) Request() (reply string, err error) {
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &jsonrpc2.Error{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				l.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	req := apis.QueryRequest{}
	req.Code = a.Code
	params, _ := json.Marshal(a.Params)
	req.Params = string(params)
	lReq := &apis.ThirdpartLogWrite{ProviderApiId: a.Provider.ProviderApiId, ProviderCode: a.Provider.ProviderApiCode}
	startTime := uint64(time.Now().UnixNano())
	defer dutils.AppendLog(a.LReqs, lReq, startTime)
	lReq.RequestParams = req.Params
	ret, err := rpc_apis.AdmData.Query(context.Background(), &req)
	if err != nil {
		var e jsonrpc2.Error
		merr := json.Unmarshal([]byte(err.Error()), &e)
		if merr != nil {
			l.Error("func",
				zap.String("call", "Query"),
				zap.String("args", utils.MarshalJsonString(req)),
				zap.String("error", err.Error()))
			lReq.ResponseParams = err.Error()
			dutils.SetAccessLogReqCodeMsgState(lReq, 1101, "gd_1101", "三方错误,请求失败", false)
		} else {

			lReq.ResponseParams = err.Error()
			if e.Code == 11004 {
				dutils.SetAccessLogReqCodeMsgState(lReq, 1100, "gd_1100", "无数据", false)
			} else {
				l.Error("func",
					zap.String("call", "Query"),
					zap.String("args", utils.MarshalJsonString(req)),
					zap.String("error", err.Error()))
				dutils.SetAccessLogReqCodeMsgState(lReq, e.Code, "gd_1101", "三方错误,请求失败", false)
			}

		}
		return "", errors.VendorError
	}
	reply = ret.Data
	lReq.ResponseParams = ret.Data
	dutils.SetAccessLogReqCodeMsgState(lReq, 0, "0", "成功", true)
	return
}
